@@ -133,9 +133,20 @@ impl Path {
 
             let normal = (n0 + n1) / 2.0;
             let length_sq = normal.length_sq();
+
+            // We can't just cut off corners for filled shapes like this,
+            // because the feather will both expand and contract the corner along the provided normals
+            // to make sure it doesn't grow, and the shrinking will make the inner points cross each other.
+            //
+            // A better approach is to shrink the vertices in by half the feather-width here
+            // and then only expand during feathering.
+            //
+            // See https://github.com/emilk/egui/issues/1226
+            const CUT_OFF_SHARP_CORNERS: bool = false;
+
             let right_angle_length_sq = 0.5;
             let sharper_than_a_right_angle = length_sq < right_angle_length_sq;
-            if sharper_than_a_right_angle {
+            if CUT_OFF_SHARP_CORNERS && sharper_than_a_right_angle {
                 // cut off the sharp corner
                 let center_normal = normal.normalized();
                 let n0c = (n0 + center_normal) / 2.0;
@@ -172,39 +183,43 @@ impl Path {
     }
 
     /// The path is taken to be closed (i.e. returning to the start again).
-    pub fn fill(&self, color: Color32, options: &TessellationOptions, out: &mut Mesh) {
-        fill_closed_path(&self.0, color, options, out);
+    ///
+    /// Calling this may reverse the vertices in the path if they are wrong winding order.
+    ///
+    /// The preferred winding order is clockwise.
+    pub fn fill(&mut self, color: Color32, options: &TessellationOptions, out: &mut Mesh) {
+        fill_closed_path(&mut self.0, color, options, out);
     }
 }
 
 pub mod path {
     //! Helpers for constructing paths
+    use crate::shape::Rounding;
+
     use super::*;
 
     /// overwrites existing points
-    pub fn rounded_rectangle(path: &mut Vec<Pos2>, rect: Rect, corner_radius: f32) {
+    pub fn rounded_rectangle(path: &mut Vec<Pos2>, rect: Rect, rounding: Rounding) {
         path.clear();
 
         let min = rect.min;
         let max = rect.max;
 
-        let cr = corner_radius
-            .min(rect.width() * 0.5)
-            .min(rect.height() * 0.5);
+        let r = clamp_radius(rounding, rect);
 
-        if cr <= 0.0 {
+        if r == Rounding::none() {
             let min = rect.min;
             let max = rect.max;
             path.reserve(4);
-            path.push(pos2(min.x, min.y));
-            path.push(pos2(max.x, min.y));
-            path.push(pos2(max.x, max.y));
-            path.push(pos2(min.x, max.y));
+            path.push(pos2(min.x, min.y)); // left top
+            path.push(pos2(max.x, min.y)); // right top
+            path.push(pos2(max.x, max.y)); // right bottom
+            path.push(pos2(min.x, max.y)); // left bottom
         } else {
-            add_circle_quadrant(path, pos2(max.x - cr, max.y - cr), cr, 0.0);
-            add_circle_quadrant(path, pos2(min.x + cr, max.y - cr), cr, 1.0);
-            add_circle_quadrant(path, pos2(min.x + cr, min.y + cr), cr, 2.0);
-            add_circle_quadrant(path, pos2(max.x - cr, min.y + cr), cr, 3.0);
+            add_circle_quadrant(path, pos2(max.x - r.se, max.y - r.se), r.se, 0.0);
+            add_circle_quadrant(path, pos2(min.x + r.sw, max.y - r.sw), r.sw, 1.0);
+            add_circle_quadrant(path, pos2(min.x + r.nw, min.y + r.nw), r.nw, 2.0);
+            add_circle_quadrant(path, pos2(max.x - r.ne, min.y + r.ne), r.ne, 3.0);
         }
     }
 
@@ -242,6 +257,20 @@ pub mod path {
             path.push(center + radius * Vec2::angled(angle));
         }
     }
+
+    // Ensures the radius of each corner is within a valid range
+    fn clamp_radius(rounding: Rounding, rect: Rect) -> Rounding {
+        let half_width = rect.width() * 0.5;
+        let half_height = rect.height() * 0.5;
+        let max_cr = half_width.min(half_height);
+
+        Rounding {
+            nw: rounding.nw.at_most(max_cr).at_least(0.0),
+            ne: rounding.ne.at_most(max_cr).at_least(0.0),
+            sw: rounding.sw.at_most(max_cr).at_least(0.0),
+            se: rounding.se.at_most(max_cr).at_least(0.0),
+        }
+    }
 }
 
 // ----------------------------------------------------------------------------
@@ -286,6 +315,12 @@ pub struct TessellationOptions {
 
     /// If true, no clipping will be done.
     pub debug_ignore_clip_rects: bool,
+
+    /// The maximum distance between the original curve and the flattened curve.
+    pub bezier_tolerance: f32,
+
+    /// The default value will be 1.0e-5, it will be used during float compare.
+    pub epsilon: f32,
 }
 
 impl Default for TessellationOptions {
@@ -299,6 +334,8 @@ impl Default for TessellationOptions {
             debug_paint_text_rects: false,
             debug_paint_clip_rects: false,
             debug_ignore_clip_rects: false,
+            bezier_tolerance: 0.1,
+            epsilon: 1.0e-5,
         }
     }
 }
@@ -324,9 +361,27 @@ impl TessellationOptions {
     }
 }
 
+fn cw_signed_area(path: &[PathPoint]) -> f64 {
+    if let Some(last) = path.last() {
+        let mut previous = last.pos;
+        let mut area = 0.0;
+        for p in path {
+            area += (previous.x * p.pos.y - p.pos.x * previous.y) as f64;
+            previous = p.pos;
+        }
+        area
+    } else {
+        0.0
+    }
+}
+
 /// Tessellate the given convex area into a polygon.
+///
+/// Calling this may reverse the vertices in the path if they are wrong winding order.
+///
+/// The preferred winding order is clockwise.
 fn fill_closed_path(
-    path: &[PathPoint],
+    path: &mut [PathPoint],
     color: Color32,
     options: &TessellationOptions,
     out: &mut Mesh,
@@ -337,14 +392,26 @@ fn fill_closed_path(
 
     let n = path.len() as u32;
     if options.anti_alias {
+        if cw_signed_area(path) < 0.0 {
+            // Wrong winding order - fix:
+            path.reverse();
+            for point in path.iter_mut() {
+                point.normal = -point.normal;
+            }
+        }
+
         out.reserve_triangles(3 * n as usize);
         out.reserve_vertices(2 * n as usize);
         let color_outer = Color32::TRANSPARENT;
         let idx_inner = out.vertices.len() as u32;
         let idx_outer = idx_inner + 1;
+
+        // The fill:
         for i in 2..n {
             out.add_triangle(idx_inner + 2 * (i - 1), idx_inner, idx_inner + 2 * i);
         }
+
+        // The feathering:
         let mut i0 = n - 1;
         for i1 in 0..n {
             let p1 = &path[i1 as usize];
@@ -710,16 +777,101 @@ impl Tessellator {
                 }
                 self.tessellate_text(tex_size, text_shape, out);
             }
+            Shape::QuadraticBezier(quadratic_shape) => {
+                self.tessellate_quadratic_bezier(quadratic_shape, out);
+            }
+            Shape::CubicBezier(cubic_shape) => self.tessellate_cubic_bezier(cubic_shape, out),
         }
     }
 
+    pub(crate) fn tessellate_quadratic_bezier(
+        &mut self,
+        quadratic_shape: QuadraticBezierShape,
+        out: &mut Mesh,
+    ) {
+        let options = &self.options;
+        let clip_rect = self.clip_rect;
+
+        if options.coarse_tessellation_culling
+            && !quadratic_shape.visual_bounding_rect().intersects(clip_rect)
+        {
+            return;
+        }
+
+        let points = quadratic_shape.flatten(Some(options.bezier_tolerance));
+
+        self.tessellate_bezier_complete(
+            &points,
+            quadratic_shape.fill,
+            quadratic_shape.closed,
+            quadratic_shape.stroke,
+            out,
+        );
+    }
+
+    pub(crate) fn tessellate_cubic_bezier(
+        &mut self,
+        cubic_shape: CubicBezierShape,
+        out: &mut Mesh,
+    ) {
+        let options = &self.options;
+        let clip_rect = self.clip_rect;
+        if options.coarse_tessellation_culling
+            && !cubic_shape.visual_bounding_rect().intersects(clip_rect)
+        {
+            return;
+        }
+
+        let points_vec =
+            cubic_shape.flatten_closed(Some(options.bezier_tolerance), Some(options.epsilon));
+
+        for points in points_vec {
+            self.tessellate_bezier_complete(
+                &points,
+                cubic_shape.fill,
+                cubic_shape.closed,
+                cubic_shape.stroke,
+                out,
+            );
+        }
+    }
+
+    fn tessellate_bezier_complete(
+        &mut self,
+        points: &[Pos2],
+        fill: Color32,
+        closed: bool,
+        stroke: Stroke,
+        out: &mut Mesh,
+    ) {
+        self.scratchpad_path.clear();
+        if closed {
+            self.scratchpad_path.add_line_loop(points);
+        } else {
+            self.scratchpad_path.add_open_points(points);
+        }
+        if fill != Color32::TRANSPARENT {
+            crate::epaint_assert!(
+                closed,
+                "You asked to fill a path that is not closed. That makes no sense."
+            );
+            self.scratchpad_path.fill(fill, &self.options, out);
+        }
+        let typ = if closed {
+            PathType::Closed
+        } else {
+            PathType::Open
+        };
+        self.scratchpad_path.stroke(typ, stroke, &self.options, out);
+    }
+
     pub(crate) fn tessellate_path(&mut self, path_shape: PathShape, out: &mut Mesh) {
         if path_shape.points.len() < 2 {
             return;
         }
 
         if self.options.coarse_tessellation_culling
-            && !path_shape.bounding_rect().intersects(self.clip_rect)
+            && !path_shape.visual_bounding_rect().intersects(self.clip_rect)
         {
             return;
         }
@@ -756,7 +908,7 @@ impl Tessellator {
     pub(crate) fn tessellate_rect(&mut self, rect: &RectShape, out: &mut Mesh) {
         let RectShape {
             mut rect,
-            corner_radius,
+            rounding,
             fill,
             stroke,
         } = *rect;
@@ -777,7 +929,7 @@ impl Tessellator {
 
         let path = &mut self.scratchpad_path;
         path.clear();
-        path::rounded_rectangle(&mut self.scratchpad_points, rect, corner_radius);
+        path::rounded_rectangle(&mut self.scratchpad_points, rect, rounding);
         path.add_line_loop(&self.scratchpad_points);
         path.fill(fill, &self.options, out);
         path.stroke_closed(stroke, &self.options, out);
@@ -920,16 +1072,20 @@ pub fn tessellate_shapes(
 
     if options.debug_paint_clip_rects {
         for ClippedMesh(clip_rect, mesh) in &mut clipped_meshes {
-            tessellator.clip_rect = Rect::EVERYTHING;
-            tessellator.tessellate_shape(
-                tex_size,
-                Shape::rect_stroke(
-                    *clip_rect,
-                    0.0,
-                    Stroke::new(2.0, Color32::from_rgb(150, 255, 150)),
-                ),
-                mesh,
-            );
+            if mesh.texture_id == TextureId::default() {
+                tessellator.clip_rect = Rect::EVERYTHING;
+                tessellator.tessellate_shape(
+                    tex_size,
+                    Shape::rect_stroke(
+                        *clip_rect,
+                        0.0,
+                        Stroke::new(2.0, Color32::from_rgb(150, 255, 150)),
+                    ),
+                    mesh,
+                );
+            } else {
+                // TODO: create a new `ClippedMesh` just for the painted clip rectangle
+            }
         }
     }
 
