@@ -1,10 +1,9 @@
 #![allow(clippy::derive_hash_xor_eq)] // We need to impl Hash for f32, but we don't implement Eq, which is fine
 
 use std::ops::Range;
-use std::sync::Arc;
 
 use super::{cursor::*, font::UvRect};
-use crate::{Color32, Mesh, Stroke, TextStyle};
+use crate::{mutex::Arc, Color32, FontId, Mesh, Stroke};
 use emath::*;
 
 /// Describes the task of laying out text.
@@ -15,14 +14,14 @@ use emath::*;
 ///
 /// ## Example:
 /// ```
-/// use epaint::{Color32, text::{LayoutJob, TextFormat}, TextStyle};
+/// use epaint::{Color32, text::{LayoutJob, TextFormat}, FontFamily, FontId};
 ///
 /// let mut job = LayoutJob::default();
 /// job.append(
 ///     "Hello ",
 ///     0.0,
 ///     TextFormat {
-///         style: TextStyle::Body,
+///         font_id: FontId::new(14.0, FontFamily::Proportional),
 ///         color: Color32::WHITE,
 ///         ..Default::default()
 ///     },
@@ -31,7 +30,7 @@ use emath::*;
 ///     "World!",
 ///     0.0,
 ///     TextFormat {
-///         style: TextStyle::Monospace,
+///         font_id: FontId::new(14.0, FontFamily::Monospace),
 ///         color: Color32::BLACK,
 ///         ..Default::default()
 ///     },
@@ -91,12 +90,12 @@ impl Default for LayoutJob {
 impl LayoutJob {
     /// Break on `\n` and at the given wrap width.
     #[inline]
-    pub fn simple(text: String, text_style: TextStyle, color: Color32, wrap_width: f32) -> Self {
+    pub fn simple(text: String, font_id: FontId, color: Color32, wrap_width: f32) -> Self {
         Self {
             sections: vec![LayoutSection {
                 leading_space: 0.0,
                 byte_range: 0..text.len(),
-                format: TextFormat::simple(text_style, color),
+                format: TextFormat::simple(font_id, color),
             }],
             text,
             wrap_width,
@@ -107,12 +106,12 @@ impl LayoutJob {
 
     /// Does not break on `\n`, but shows the replacement character instead.
     #[inline]
-    pub fn simple_singleline(text: String, text_style: TextStyle, color: Color32) -> Self {
+    pub fn simple_singleline(text: String, font_id: FontId, color: Color32) -> Self {
         Self {
             sections: vec![LayoutSection {
                 leading_space: 0.0,
                 byte_range: 0..text.len(),
-                format: TextFormat::simple(text_style, color),
+                format: TextFormat::simple(font_id, color),
             }],
             text,
             wrap_width: f32::INFINITY,
@@ -157,7 +156,7 @@ impl LayoutJob {
     pub fn font_height(&self, fonts: &crate::Fonts) -> f32 {
         let mut max_height = 0.0_f32;
         for section in &self.sections {
-            max_height = max_height.max(fonts.row_height(section.format.style));
+            max_height = max_height.max(fonts.row_height(&section.format.font_id));
         }
         max_height
     }
@@ -214,10 +213,10 @@ impl std::hash::Hash for LayoutSection {
 
 // ----------------------------------------------------------------------------
 
-#[derive(Copy, Clone, Debug, Hash, PartialEq)]
+#[derive(Clone, Debug, Hash, PartialEq)]
 #[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct TextFormat {
-    pub style: TextStyle,
+    pub font_id: FontId,
     /// Text color
     pub color: Color32,
     pub background: Color32,
@@ -234,7 +233,7 @@ impl Default for TextFormat {
     #[inline]
     fn default() -> Self {
         Self {
-            style: TextStyle::Body,
+            font_id: FontId::default(),
             color: Color32::GRAY,
             background: Color32::TRANSPARENT,
             italics: false,
@@ -247,9 +246,9 @@ impl Default for TextFormat {
 
 impl TextFormat {
     #[inline]
-    pub fn simple(style: TextStyle, color: Color32) -> Self {
+    pub fn simple(font_id: FontId, color: Color32) -> Self {
         Self {
-            style,
+            font_id,
             color,
             ..Default::default()
         }
@@ -262,6 +261,7 @@ impl TextFormat {
 ///
 /// You can create a [`Galley`] using [`crate::Fonts::layout_job`];
 #[derive(Clone, Debug, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct Galley {
     /// The job that this galley is the result of.
     /// Contains the original string and style sections.
@@ -295,6 +295,7 @@ pub struct Galley {
 }
 
 #[derive(Clone, Debug, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct Row {
     /// One for each `char`.
     pub glyphs: Vec<Glyph>,
@@ -317,6 +318,7 @@ pub struct Row {
 
 /// The tessellated output of a row.
 #[derive(Clone, Debug, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct RowVisuals {
     /// The tessellated text, using non-normalized (texel) UV coordinates.
     /// That is, you need to divide the uv coordinates by the texture size.
@@ -342,6 +344,7 @@ impl Default for RowVisuals {
 }
 
 #[derive(Copy, Clone, Debug, PartialEq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
 pub struct Glyph {
     pub chr: char,
     /// Relative to the galley position.
